package ru.dbotthepony.mc.otm.datagen

import net.minecraft.resources.ResourceLocation
import net.minecraft.world.level.block.Block
import net.minecraftforge.client.model.generators.BlockStateProvider
import net.minecraftforge.eventbus.api.SubscribeEvent
import net.minecraftforge.fml.common.Mod
import net.minecraftforge.forge.event.lifecycle.GatherDataEvent
import ru.dbotthepony.mc.otm.OverdriveThatMatters
import ru.dbotthepony.mc.otm.Registry.Blocks
import ru.dbotthepony.mc.otm.Registry.Items
import ru.dbotthepony.mc.otm.block.BlockDriveViewer
import ru.dbotthepony.mc.otm.block.BlockMatteryRotatable
import ru.dbotthepony.mc.otm.block.BlockPatternStorage
import ru.dbotthepony.mc.otm.block.entity.worker.WorkerState
import ru.dbotthepony.mc.otm.datagen.blocks.*
import ru.dbotthepony.mc.otm.datagen.items.SimpleItemModelProvider
import ru.dbotthepony.mc.otm.datagen.loot.MatteryLootTableProvider
import ru.dbotthepony.mc.otm.datagen.loot.TileNbtCopy
import ru.dbotthepony.mc.otm.datagen.models.BlockMatteryModelProvider

@Mod.EventBusSubscriber(modid = DataGen.MOD_ID, bus = Mod.EventBusSubscriber.Bus.MOD)
object DataGen {
	const val MOD_ID = OverdriveThatMatters.MOD_ID

	private lateinit var blockModelProvider: BlockMatteryModelProvider
	private lateinit var itemModelProvider: SimpleItemModelProvider
	private lateinit var blockStateProvider: MatteryBlockStateProvider
	private lateinit var lootTableProvider: MatteryLootTableProvider

	private fun decorativeCubeAll(vararg blocks: Block) {
		blockModelProvider.decorativeCubeAll(*blocks)
		blockStateProvider.block(*blocks)
	}

	private fun resourceCubeAll(vararg blocks: Block) {
		blockModelProvider.resourceCubeAll(*blocks)
		blockStateProvider.block(*blocks)
	}

	private fun decorativeColumn(block: Block, end: String, side: String) {
		blockModelProvider.decorativeColumn(block, end, side)
		blockStateProvider.block(block)
	}

	@SubscribeEvent
	@JvmStatic
	@Suppress("unused")
	fun onGatherData(event: GatherDataEvent) {
		blockModelProvider = BlockMatteryModelProvider(event)
		blockStateProvider = MatteryBlockStateProvider(event)
		itemModelProvider = SimpleItemModelProvider(event)
		lootTableProvider = MatteryLootTableProvider(event.generator)

		event.generator.addProvider(blockModelProvider)
		event.generator.addProvider(itemModelProvider)
		event.generator.addProvider(blockStateProvider)
		event.generator.addProvider(blockStateProvider)

		decorativeCubeAll(*Blocks.CRATES)
		decorativeCubeAll(Blocks.CARBON_FIBRE_BLOCK)
		decorativeCubeAll(Blocks.TRITANIUM_BLOCK)

		blockModelProvider.resourceCubeAll(Blocks.TRITANIUM_ORE)
		blockModelProvider.resourceCubeAll(Blocks.TRITANIUM_RAW_BLOCK)
		blockModelProvider.resourceCubeAll(Blocks.DEEPSLATE_TRITANIUM_ORE)

		decorativeColumn(Blocks.TRITANIUM_STRIPED_BLOCK, "tritanium_block", "tritanium_striped_block")

		with(blockModelProvider) {
			lambda {
				copy("block/battery/battery1", "block/battery/battery0").also { it.offset(-4f, 0f, 0f) }
				copy("block/battery/battery2", "block/battery/battery0").also { it.offset(-8f, 0f, 0f) }

				copy("block/battery/battery3", "block/battery/battery0").also { it.offset(0f, 6f, 0f) }
				copy("block/battery/battery4", "block/battery/battery0").also { it.offset(-4f, 6f, 0f) }
				copy("block/battery/battery5", "block/battery/battery0").also { it.offset(-8f, 6f, 0f) }

				copy("block/battery/battery7", "block/battery/battery6").also { it.offset(4f, 0f, 0f) }
				copy("block/battery/battery8", "block/battery/battery6").also { it.offset(8f, 0f, 0f) }

				copy("block/battery/battery9", "block/battery/battery6").also { it.offset(0f, 6f, 0f) }
				copy("block/battery/battery10", "block/battery/battery6").also { it.offset(4f, 6f, 0f) }
				copy("block/battery/battery11", "block/battery/battery6").also { it.offset(8f, 6f, 0f) }

				for (i in 0 .. 11) {
					withExistingParent("block/battery/matter_capacitor$i", ResourceLocation(MOD_ID, "block/battery/battery$i"))
						.texture("1", "block/matterybank_core")
				}

				for (i in 1 .. 7) {
					copy("block/pattern/model$i", "block/pattern/model0").also { it.offset(-2f * i, 0f, 0f) }
				}
			}
		}

		with(blockStateProvider) {
			block(Blocks.BLACK_HOLE)
			block(Blocks.ANDROID_STATION)

			ore(Blocks.DEEPSLATE_TRITANIUM_ORE)
			ore(Blocks.TRITANIUM_ORE)
			ore(Blocks.TRITANIUM_RAW_BLOCK)
		}

		event.generator.addProvider(MatterBankProvider(event))
		event.generator.addProvider(BatteryBankProvider(event))

		event.generator.addProvider(lootTableProvider)

		with(blockStateProvider) {
			block(Blocks.CHEMICAL_GENERATOR)
			block(Blocks.MATTER_SCANNER)
			block(Blocks.ITEM_MONITOR)
			block(Blocks.MATTER_BOTTLER)
			block(Blocks.MATTER_DECOMPOSER)
			block(Blocks.MATTER_REPLICATOR)
		}

		with(itemModelProvider) {
			block(Items.ANDROID_STATION)
			block(Items.BATTERY_BANK)
			block(Items.MATTER_CAPACITOR_BANK)
			block(Items.PATTERN_STORAGE)
			block(Items.BLACK_HOLE)
			block(Items.CARBON_FIBRE_BLOCK)
			block(Items.DEEPSLATE_TRITANIUM_ORE)
			block(Items.TRITANIUM_ORE)
			block(Items.TRITANIUM_STRIPED_BLOCK)
			block(Items.TRITANIUM_RAW_BLOCK)
			block(Items.ITEM_MONITOR)

			blocks(*Items.CRATES)

			components(*Items.DATAGEN_COMPONENTS)

			generated(Items.PILL_ANDROID)
			generated(Items.PILL_HUMANE)
			generated(Items.NUTRIENT_PASTE)
			component(Items.TRITANIUM_INGOT)
			resource(Items.TRITANIUM_ORE_CLUMP)

			generatedTiered(Items.BATTERIES, "battery_tier")
			generated(Items.BATTERY_CREATIVE)

			generated(Items.MATTER_CAPACITOR_BASIC, ResourceLocation(MOD_ID, "item/matter_capacitor_tier1"))
			generated(Items.MATTER_CAPACITOR_NORMAL, ResourceLocation(MOD_ID, "item/matter_capacitor_tier2"))
			generated(Items.MATTER_CAPACITOR_DENSE, ResourceLocation(MOD_ID, "item/matter_capacitor_tier3"))
			generated(Items.MATTER_CAPACITOR_CREATIVE)

			generated(Items.PATTERN_DRIVE_CREATIVE)

			block(Items.CARGO_CRATE, "cargo_crate_closed")
			block(Items.CHEMICAL_GENERATOR, "chemical_generator_working")
			block(Items.ENERGY_COUNTER, "energy_counter_down")
			block(Items.MATTER_BOTTLER, "matter_bottler_working")
			block(Items.MATTER_CABLE, "matter_cable_core")
			block(Items.MATTER_DECOMPOSER, "matter_decomposer_working")
		}

		blockStateProvider.lambda {
			with(it.getMultipartBuilder(Blocks.DRIVE_VIEWER)) {
				for (facing in BlockMatteryRotatable.FACING.possibleValues) {
					part()
						.modelFile(it.models().getExistingFile(ResourceLocation(MOD_ID, "block/drive_viewer_drive_part")))
						.rotationY(facing.toYRotBlockstate())
						.addModel()
						.condition(BlockMatteryRotatable.FACING, facing)
						.condition(BlockDriveViewer.DRIVE_PRESENT, true)

					for (workState in WorkerState.SEMI_WORKER_STATE.possibleValues) {
						part()
							.modelFile(it.models().getExistingFile(ResourceLocation(MOD_ID, "block/drive_viewer_${workState.name.lowercase()}")))
							.rotationY(facing.toYRotBlockstate())
							.addModel()
							.condition(WorkerState.SEMI_WORKER_STATE, workState)
							.condition(BlockMatteryRotatable.FACING, facing)
					}
				}
			}

			with(it.getMultipartBuilder(Blocks.PATTERN_STORAGE)) {
				for (facing in BlockMatteryRotatable.FACING.possibleValues) {
					for (i in 0 .. 7) {
						part()
							.modelFile(it.models().getExistingFile(ResourceLocation(MOD_ID, "block/pattern/model$i")))
							.rotationY(facing.toYRotBlockstate())
							.addModel()
							.condition(BlockMatteryRotatable.FACING, facing)
							.condition(BlockPatternStorage.PATTERN_STORAGE_DISKS_PROPS[i], true)
					}

					part()
						.modelFile(it.models().getExistingFile(ResourceLocation(MOD_ID, "block/pattern_storage")))
						.rotationY(facing.toYRotBlockstate())
						.addModel()
						.condition(BlockMatteryRotatable.FACING, facing)
				}
			}
		}

		val workerTags = arrayOf(
			TileNbtCopy("work_ticks"),
			TileNbtCopy("current_job"),
			TileNbtCopy("battery_container"),
			TileNbtCopy("energy_cap"),
		)

		with(lootTableProvider) {
			simpleBlocks(*Blocks.CRATES)
			simpleBlock(Blocks.CARGO_CRATE)
			simpleBlock(Blocks.CARBON_FIBRE_BLOCK)
			simpleBlock(Blocks.TRITANIUM_RAW_BLOCK)
			simpleBlock(Blocks.TRITANIUM_BLOCK)
			simpleBlock(Blocks.TRITANIUM_STRIPED_BLOCK)
			simpleBlock(Blocks.MATTER_CABLE)

			tile(Blocks.ENERGY_COUNTER, TileNbtCopy("passed"), TileNbtCopy("history"), TileNbtCopy("history_tick"))
			tile(Blocks.CHEMICAL_GENERATOR, TileNbtCopy("energy"), TileNbtCopy("slots"), TileNbtCopy("working_ticks"), TileNbtCopy("working_ticks_total"))

			tile(Blocks.ANDROID_STATION, TileNbtCopy("energy_cap"), TileNbtCopy("battery_container"))
			tile(Blocks.BATTERY_BANK, TileNbtCopy("battery_bank"))
			tile(Blocks.DRIVE_VIEWER, TileNbtCopy("energy_cap"), TileNbtCopy("drive_slot"), TileNbtCopy("battery_container"))

			tile(Blocks.MATTER_DECOMPOSER, TileNbtCopy("work_slots"), TileNbtCopy("matter_capability"), *workerTags)
			tile(Blocks.MATTER_REPLICATOR, TileNbtCopy("regular_slots"), TileNbtCopy("reserved_slots"), TileNbtCopy("matter_capability"), *workerTags)
			tile(Blocks.MATTER_SCANNER, TileNbtCopy("work_slots"), *workerTags)
			tile(Blocks.MATTER_PANEL, TileNbtCopy("tasks"))
			tile(Blocks.PATTERN_STORAGE, TileNbtCopy("patterns"))
			tile(Blocks.MATTER_CAPACITOR_BANK, TileNbtCopy("matter_container"))
			tile(Blocks.MATTER_BOTTLER, TileNbtCopy("energy_cap"), TileNbtCopy("battery_container"), TileNbtCopy("work_slots"), TileNbtCopy("work_flow"), TileNbtCopy("matter_capability"))
		}
	}
}
