package ru.dbotthepony.mc.otm.datagen.blocks

import net.minecraft.resources.ResourceLocation
import net.minecraft.world.level.block.Block
import net.minecraft.world.level.block.state.BlockState
import net.minecraftforge.client.model.generators.BlockStateProvider
import net.minecraftforge.client.model.generators.ConfiguredModel
import net.minecraftforge.forge.event.lifecycle.GatherDataEvent
import ru.dbotthepony.mc.otm.block.BlockMattery
import ru.dbotthepony.mc.otm.block.BlockMatteryRotatable
import ru.dbotthepony.mc.otm.block.entity.worker.WorkerState
import ru.dbotthepony.mc.otm.datagen.DataGen
import ru.dbotthepony.mc.otm.datagen.getValueNullable
import ru.dbotthepony.mc.otm.datagen.toYRotBlockstate

typealias AdvancedBlockStateFunction = (BlockState, ConfiguredModel.Builder<*>, String) -> String?
private data class AdvancedBlockStateEntry(val block: BlockMattery, val func: AdvancedBlockStateFunction)

private fun initialTransform(it: BlockState, modelPath: String, builder: ConfiguredModel.Builder<*>): String {
	var modelPath = modelPath

	it.getValueNullable(BlockMatteryRotatable.FACING)?.let {
		builder.rotationY(it.toYRotBlockstate())
	}

	it.getValueNullable(WorkerState.WORKER_STATE)?.let {
		modelPath += "_" + it.name.lowercase()
	}

	it.getValueNullable(WorkerState.SEMI_WORKER_STATE)?.let {
		modelPath += "_" + it.name.lowercase()
	}

	return modelPath
}

private typealias Lambda = (MatteryBlockStateProvider) -> Unit

class MatteryBlockStateProvider(event: GatherDataEvent) : BlockStateProvider(event.generator, DataGen.MOD_ID, event.existingFileHelper) {
	private val blocks = ArrayList<AdvancedBlockStateEntry>()

	fun block(block: BlockMattery) = blocks.add(AdvancedBlockStateEntry(block) { _, _, _ -> null })
	fun block(block: BlockMattery, func: AdvancedBlockStateFunction) = blocks.add(AdvancedBlockStateEntry(block, func))

	private val simpleBlocks = ArrayList<Block>()
	private val ores = ArrayList<Block>()
	private val lambdas = ArrayList<Lambda>()

	fun block(vararg blocks: Block) {
		this.simpleBlocks.addAll(blocks)
	}

	fun ore(vararg blocks: Block) {
		ores.addAll(blocks)
	}

	fun lambda(lambda: Lambda) {
		lambdas.add(lambda)
	}

	override fun registerStatesAndModels() {
		for (block in simpleBlocks) {
			getVariantBuilder(block).forAllStates {
				check(block.registryName != null) {"$block registry name is null!"}
				return@forAllStates arrayOf(ConfiguredModel(models().getExistingFile(block.registryName)))
			}
		}

		for (block in ores) {
			var first = true

			getVariantBuilder(block).forAllStates {
				check(first) {"$block has more than one BlockState"}
				first = false

				check(block.registryName != null) {"$block registry name is null!"}
				val localModel = models().getExistingFile(block.registryName)

				val generated = ArrayList<ConfiguredModel>()

				for (x in arrayOf(0, 90, 180)) {
					for (y in arrayOf(0, 90, 180, 270)) {
						generated.add(with(ConfiguredModel.builder()) {
							rotationX(x)
							rotationY(y)
							modelFile(localModel)
							return@with buildLast()
						})
					}
				}

				return@forAllStates generated.toTypedArray()
			}
		}

		for ((block, func) in blocks) {
			getVariantBuilder(block).forAllStates {
				val builder = ConfiguredModel.builder()
				var modelPath = initialTransform(it, "block/${block.registryName!!.path}", builder)
				modelPath = func(it, builder, modelPath) ?: modelPath

				builder.modelFile(models().getExistingFile(ResourceLocation(DataGen.MOD_ID, modelPath)))

				return@forAllStates arrayOf(builder.buildLast())
			}
		}

		for (lambda in lambdas) {
			lambda(this)
		}
	}
}
