package ru.dbotthepony.mc.otm.datagen.items

import net.minecraft.resources.ResourceLocation
import net.minecraft.world.item.BlockItem
import net.minecraft.world.item.Item
import net.minecraftforge.client.model.generators.ItemModelProvider
import net.minecraftforge.forge.event.lifecycle.GatherDataEvent
import ru.dbotthepony.mc.otm.datagen.DataGen

private data class ItemModelGenerated(val item: String, val path: ResourceLocation)
private data class ItemModelDelegate(val item: String, val path: ResourceLocation)

class SimpleItemModelProvider(event: GatherDataEvent) : ItemModelProvider(event.generator, DataGen.MOD_ID, event.existingFileHelper) {
	private val generated = ArrayList<ItemModelGenerated>()
	private val delegates = ArrayList<ItemModelDelegate>()

	override fun registerModels() {
		for ((item, path) in generated) {
			withExistingParent(item, GENERATED).texture("layer0", path)
		}

		for ((item, path) in delegates) {
			withExistingParent(item, path)
		}
	}

	fun block(item: Item) = delegates.add(ItemModelDelegate(item.registryName!!.path, ResourceLocation(DataGen.MOD_ID, "block/${item.registryName!!.path}")))
	fun block(item: Item, path: String) = delegates.add(ItemModelDelegate(item.registryName!!.path, ResourceLocation(DataGen.MOD_ID, "block/$path")))
	fun blocks(vararg items: Item) = items.forEach(this::block)

	fun generated(item: Item, texture: ResourceLocation) {
		generated.add(ItemModelGenerated(item.registryName!!.path, texture))
	}

	fun generated(item: Item) = generated(item, ResourceLocation(DataGen.MOD_ID, "item/${item.registryName!!.path}"))
	fun generated(item: Item, prefix: String) = generated(item, ResourceLocation(DataGen.MOD_ID, "item/${prefix}${item.registryName!!.path}"))
	fun component(item: Item) = generated(item, "component/")
	fun components(vararg items: Item) = items.forEach(this::component)
	fun resource(item: Item) = generated(item, "resources/")
	fun resources(vararg items: Item) = items.forEach(this::resource)

	fun generatedTiered(items: Array<out Item>, prefix: String) {
		var i = 0

		for (item in items) {
			generated(item, ResourceLocation(DataGen.MOD_ID, "item/$prefix$i"))
			i++
		}
	}

	companion object {
		private val GENERATED = ResourceLocation("minecraft", "item/generated")
	}
}
