package ru.dbotthepony.mc.otm.datagen.loot

import com.mojang.datafixers.util.Pair
import net.minecraft.data.DataGenerator
import net.minecraft.data.loot.LootTableProvider
import net.minecraft.resources.ResourceLocation
import net.minecraft.world.level.block.Block
import net.minecraft.world.level.storage.loot.LootPool
import net.minecraft.world.level.storage.loot.LootTable
import net.minecraft.world.level.storage.loot.ValidationContext
import net.minecraft.world.level.storage.loot.entries.LootItem
import net.minecraft.world.level.storage.loot.functions.CopyNbtFunction
import net.minecraft.world.level.storage.loot.functions.LootItemFunctions
import net.minecraft.world.level.storage.loot.parameters.LootContextParamSet
import net.minecraft.world.level.storage.loot.parameters.LootContextParamSets
import net.minecraft.world.level.storage.loot.providers.nbt.ContextNbtProvider
import java.util.function.BiConsumer
import java.util.function.Consumer
import java.util.function.Supplier

typealias LootBuilderCallback = BiConsumer<ResourceLocation, LootTable.Builder>
typealias LootBuilderWorker = Consumer<LootBuilderCallback>
typealias LootBuilderWorkerSupplier = Supplier<LootBuilderWorker>
typealias LootTuple = Pair<LootBuilderWorkerSupplier, LootContextParamSet>

fun singleLootPool(f: (LootPool.Builder) -> Unit): LootTable.Builder {
	return LootTable.lootTable().withPool(LootPool.lootPool().also(f))
}

data class NbtCopy(val source: String, val destination: String, val strategy: CopyNbtFunction.MergeStrategy = CopyNbtFunction.MergeStrategy.REPLACE)
data class TileNbtCopy(val source: String, val strategy: CopyNbtFunction.MergeStrategy = CopyNbtFunction.MergeStrategy.REPLACE)

class MatteryLootTableProvider(generator: DataGenerator) : LootTableProvider(generator) {
	private val providers = ArrayList<LootTuple>()

	override fun getTables(): List<LootTuple> {
		return providers
	}

	override fun validate(map: MutableMap<ResourceLocation, LootTable>, validationtracker: ValidationContext) {}

	fun block(provider: LootBuilderWorker) {
		providers.add(Pair(Supplier {provider}, LootContextParamSets.BLOCK))
	}

	fun simpleBlock(block: Block) {
		block {
			it.accept(block.lootTable, singleLootPool {
				it.add(LootItem.lootTableItem(block))
			})
		}
	}

	fun simpleBlocks(vararg blocks: Block) {
		blocks.forEach(this::simpleBlock)
	}

	fun tile(block: Block, f: (CopyNbtFunction.Builder) -> Unit = {}) {
		block {
			it.accept(block.lootTable, singleLootPool {
				it.add(LootItem.lootTableItem(block).also {
					it.apply(CopyNbtFunction.copyData(ContextNbtProvider.BLOCK_ENTITY).also {
						it.copy("Name", "BlockEntityTag.Name")
						f(it)
					})
				})
			})
		}
	}

	fun tile(block: Block, vararg tags: NbtCopy) {
		tile(block) {
			for ((source, destination, strategy) in tags) {
				it.copy(source, destination, strategy)
			}
		}
	}

	fun tile(block: Block, vararg tags: TileNbtCopy) {
		tile(block) {
			for ((source, strategy) in tags) {
				it.copy(source, "BlockEntityTag.$source", strategy)
			}
		}
	}
}
