package ru.dbotthepony.mc.otm.block

import net.minecraft.core.BlockPos
import net.minecraft.core.Direction
import net.minecraft.server.level.ServerLevel
import net.minecraft.world.level.BlockGetter
import net.minecraft.world.level.Level
import net.minecraft.world.level.block.Block
import net.minecraft.world.level.block.EntityBlock
import net.minecraft.world.level.block.entity.BlockEntity
import net.minecraft.world.level.block.entity.BlockEntityTicker
import net.minecraft.world.level.block.entity.BlockEntityType
import net.minecraft.world.level.block.state.BlockState
import net.minecraft.world.level.block.state.StateDefinition
import net.minecraft.world.phys.shapes.CollisionContext
import net.minecraft.world.phys.shapes.VoxelShape
import ru.dbotthepony.mc.otm.OverdriveThatMatters
import ru.dbotthepony.mc.otm.Registry
import ru.dbotthepony.mc.otm.block.entity.BlockEntityChemicalGenerator
import ru.dbotthepony.mc.otm.block.entity.worker.WorkerState
import ru.dbotthepony.mc.otm.shapes.BlockShapes

class BlockChemicalGenerator : BlockMatteryRotatable(), EntityBlock {
	override fun newBlockEntity(p_153215_: BlockPos, p_153216_: BlockState): BlockEntity {
		return BlockEntityChemicalGenerator(p_153215_, p_153216_)
	}

	override fun <T : BlockEntity?> getTicker(
		level: Level,
		state: BlockState,
		type: BlockEntityType<T>
	): BlockEntityTicker<T>? {
		if (level.isClientSide || type != Registry.BlockEntities.CHEMICAL_GENERATOR)
			return null

		return BlockEntityTicker<T> { _, _, _, tile -> if (tile is BlockEntityChemicalGenerator) tile.tick() }
	}

	override fun createBlockStateDefinition(builder: StateDefinition.Builder<Block, BlockState>) {
		super.createBlockStateDefinition(builder)
		builder.add(WorkerState.SEMI_WORKER_STATE)
	}

	override fun neighborChanged(
		state: BlockState,
		level: Level,
		pos: BlockPos,
		sender: Block,
		sender_pos: BlockPos,
		flag: Boolean
	) {
		super.neighborChanged(state, level, pos, sender, sender_pos, flag)

		if (!level.isClientSide && level is ServerLevel) {
			val tile = level.getBlockEntity(pos)

			if (tile is BlockEntityChemicalGenerator) {
				OverdriveThatMatters.tickOnce(level) {
					tile.checkSurroundings()
				}
			}
		}
	}

	override fun getShape(
		p_60555_: BlockState,
		p_60556_: BlockGetter,
		p_60557_: BlockPos,
		p_60558_: CollisionContext
	): VoxelShape {
		return SHAPES[p_60555_.getValue(FACING).ordinal]
	}

	companion object {
		private val def = BlockShapes.CHEMICAL_GENERATOR.computeShape()

		private val SHAPES: List<VoxelShape> = listOf(
			def,
			def,
			def,
			BlockShapes.CHEMICAL_GENERATOR.rotate(Direction.NORTH).computeShape(),
			BlockShapes.CHEMICAL_GENERATOR.rotate(Direction.WEST).computeShape(),
			BlockShapes.CHEMICAL_GENERATOR.rotate(Direction.EAST).computeShape()
		)
	}
}
