package ru.dbotthepony.mc.otm.block

import net.minecraft.core.BlockPos
import net.minecraft.core.Direction
import net.minecraft.world.InteractionHand
import net.minecraft.world.InteractionResult
import net.minecraft.world.MenuProvider
import net.minecraft.world.entity.LivingEntity
import net.minecraft.world.entity.player.Player
import net.minecraft.world.item.ItemStack
import net.minecraft.world.item.context.BlockPlaceContext
import net.minecraft.world.level.Level
import net.minecraft.world.level.block.Block
import net.minecraft.world.level.block.EntityBlock
import net.minecraft.world.level.block.state.BlockState
import net.minecraft.world.level.block.state.StateDefinition
import net.minecraft.world.level.block.state.properties.EnumProperty
import net.minecraft.world.level.material.Material
import net.minecraft.world.level.material.MaterialColor
import net.minecraft.world.phys.BlockHitResult
import ru.dbotthepony.mc.otm.block.entity.BlockEntityMattery

abstract class BlockMattery @JvmOverloads constructor(
	properties: Properties = DEFAULT_PROPERTIES
) : Block(properties) {
	override fun setPlacedBy(
		level: Level,
		blockPos: BlockPos,
		blockState: BlockState,
		entity: LivingEntity?,
		itemStack: ItemStack
	) {
		if (this is EntityBlock && itemStack.hasCustomHoverName() && !level.isClientSide) {
			val tile = level.getBlockEntity(blockPos)

			if (tile is BlockEntityMattery)
				tile.customDisplayName = itemStack.displayName
		}

		super.setPlacedBy(level, blockPos, blockState, entity, itemStack)
	}

	override fun use(
		blockState: BlockState,
		level: Level,
		blockPos: BlockPos,
		ply: Player,
		hand: InteractionHand,
		blockHitResult: BlockHitResult
	): InteractionResult {
		if (this is EntityBlock && !level.isClientSide) {
			val tile = level.getBlockEntity(blockPos)

			if (tile is MenuProvider) {
				ply.openMenu(tile)
				return InteractionResult.CONSUME
			}
		}

		if (this is EntityBlock && level.isClientSide)
			return InteractionResult.SUCCESS

		return super.use(blockState, level, blockPos, ply, hand, blockHitResult)
	}

	override fun neighborChanged(
		state: BlockState,
		level: Level,
		pos: BlockPos,
		sender: Block,
		sender_pos: BlockPos,
		flag: Boolean
	) {
		super.neighborChanged(state, level, pos, sender, sender_pos, flag)

		if (this is EntityBlock && !level.isClientSide) {
			val tile = level.getBlockEntity(pos)

			if (tile is BlockEntityMattery)
				tile.redstoneSignal = level.getBestNeighborSignal(pos)
		}
	}

	companion object {
		@JvmField
		val DEFAULT_PROPERTIES: Properties = Properties.of(Material.STONE, MaterialColor.METAL).requiresCorrectToolForDrops().strength(1.5f, 25.0f)
	}
}

abstract class BlockMatteryRotatable @JvmOverloads constructor(properties: Properties = DEFAULT_PROPERTIES) : BlockMattery(properties) {
	init {
		registerDefaultState(getStateDefinition().any().setValue(FACING, Direction.SOUTH))
	}

	override fun createBlockStateDefinition(builder: StateDefinition.Builder<Block, BlockState>) {
		builder.add(FACING)
	}

	override fun getStateForPlacement(context: BlockPlaceContext): BlockState? {
		return defaultBlockState().setValue(
			FACING,
			if (faceToPlayer(context)) context.horizontalDirection.opposite else context.horizontalDirection
		)
	}

	open fun faceToPlayer(context: BlockPlaceContext): Boolean {
		return true
	}

	companion object {
		@JvmField
		val FACING = EnumProperty.create(
			"facing",
			Direction::class.java,
			Direction.SOUTH,
			Direction.WEST,
			Direction.NORTH,
			Direction.EAST
		)
	}
}
