package ru.dbotthepony.mc.otm.block.entity

import net.minecraft.MethodsReturnNonnullByDefault
import net.minecraft.core.BlockPos
import net.minecraft.nbt.CompoundTag
import net.minecraft.network.chat.Component
import net.minecraft.network.chat.TranslatableComponent
import net.minecraft.world.entity.player.Inventory
import net.minecraft.world.entity.player.Player
import net.minecraft.world.inventory.AbstractContainerMenu
import net.minecraft.world.item.ItemStack
import net.minecraft.world.level.block.Block
import net.minecraft.world.level.block.state.BlockState
import ru.dbotthepony.mc.otm.OverdriveThatMatters
import ru.dbotthepony.mc.otm.Registry
import ru.dbotthepony.mc.otm.block.BlockDriveViewer
import ru.dbotthepony.mc.otm.block.entity.worker.WorkerState
import ru.dbotthepony.mc.otm.capability.MatteryCapability
import ru.dbotthepony.mc.otm.capability.MatteryMachineEnergyStorage
import ru.dbotthepony.mc.otm.container.MatteryContainer
import ru.dbotthepony.mc.otm.core.Fraction
import ru.dbotthepony.mc.otm.ifHas
import ru.dbotthepony.mc.otm.menu.DriveViewerMenu
import ru.dbotthepony.mc.otm.set
import javax.annotation.ParametersAreNonnullByDefault

@MethodsReturnNonnullByDefault
@ParametersAreNonnullByDefault
class BlockEntityDriveViewer(p_155229_: BlockPos, p_155230_: BlockState) : BlockEntityMatteryPowered(Registry.BlockEntities.DRIVE_VIEWER, p_155229_, p_155230_) {
	override fun setChanged() {
		super.setChanged()

		val level = level

		if (level != null) {
			OverdriveThatMatters.tickOnceSelf(level) {
				if (isRemoved) return@tickOnceSelf

				var state = blockState

				if (drive_slot.getItem(0).getCapability(MatteryCapability.DRIVE).isPresent && energy.batteryLevel >= OverdriveThatMatters.ITEM_STORAGE.energyPerOperation()) {
					state = state.setValue(WorkerState.SEMI_WORKER_STATE, WorkerState.WORKING)
				} else {
					state = state.setValue(WorkerState.SEMI_WORKER_STATE, WorkerState.IDLE)
				}

				if (state !== blockState) {
					it.setBlock(blockPos, state, Block.UPDATE_CLIENTS)
				}
			}
		}
	}

	@JvmField
	val drive_slot: MatteryContainer = object : MatteryContainer(this::setChanged, 1) {
		override fun setChanged(slot: Int, new: ItemStack, old: ItemStack) {
			super.setChanged(slot, new, old)

			val level = level
			if (level != null) {
				OverdriveThatMatters.tickOnceSelf(level) {
					if (!isRemoved) {
						var state = blockState

						if (new.getCapability(MatteryCapability.DRIVE).isPresent) {
							state = state.setValue(BlockDriveViewer.DRIVE_PRESENT, true)
						} else {
							state = state.setValue(BlockDriveViewer.DRIVE_PRESENT, false)
						}

						if (state !== blockState) {
							level.setBlock(blockPos, state, Block.UPDATE_CLIENTS)
						}
					}
				}
			}
		}
	}

	init {
		energy = MatteryMachineEnergyStorage(this, MatteryMachineEnergyStorage.MachineType.WORKER, Fraction(30000))
	}

	override fun getDefaultDisplayName(): Component {
		return NAME
	}

	override fun createMenu(containerID: Int, inventory: Inventory, ply: Player): AbstractContainerMenu? {
		return DriveViewerMenu(containerID, inventory, this)
	}

	override fun saveAdditional(nbt: CompoundTag) {
		super.saveAdditional(nbt)
		nbt["drive_slot"] = drive_slot.serializeNBT()
	}

	override fun load(nbt: CompoundTag) {
		super.load(nbt)

		nbt.ifHas("drive_slot") {
			drive_slot.deserializeNBT(it)
		}
	}

	fun tick() {
		batteryChargeLoop()
	}

	companion object {
		private val NAME = TranslatableComponent("block.overdrive_that_matters.drive_viewer")
	}
}