package ru.dbotthepony.mc.otm.capability

import net.minecraftforge.energy.IEnergyStorage
import ru.dbotthepony.mc.otm.core.Fraction

// IEnergyStorage for direct compat with Forge Energy
interface IMatteryEnergyStorage : IEnergyStorage {
	// such as cables. This is something that would work only with energy storage
	fun extractEnergyOuter(howMuch: Fraction, simulate: Boolean): Fraction

	// for internal needs, e.g. for work
	// CAN also be used by something that does evil
	// e.g. sucking out energy anomaly should use this
	fun extractEnergyInner(howMuch: Fraction, simulate: Boolean): Fraction

	// energy is received from outside, e.g. cables
	fun receiveEnergyOuter(howMuch: Fraction, simulate: Boolean): Fraction

	// energy is received from inside, e.g. generator generates power
	fun receiveEnergyInner(howMuch: Fraction, simulate: Boolean): Fraction

	fun extractEnergyOuter(howMuch: Long, simulate: Boolean): Fraction {
		return extractEnergyOuter(Fraction(howMuch), simulate)
	}

	fun extractEnergyOuter(howMuch: Int, simulate: Boolean): Fraction {
		return extractEnergyOuter(Fraction(howMuch), simulate)
	}

	fun receiveEnergyOuter(howMuch: Long, simulate: Boolean): Fraction {
		return receiveEnergyOuter(Fraction(howMuch), simulate)
	}

	fun receiveEnergyOuter(howMuch: Int, simulate: Boolean): Fraction {
		return receiveEnergyOuter(Fraction(howMuch), simulate)
	}

	fun extractEnergyInner(howMuch: Long, simulate: Boolean): Fraction {
		return extractEnergyInner(Fraction(howMuch), simulate)
	}

	fun extractEnergyInner(howMuch: Int, simulate: Boolean): Fraction {
		return extractEnergyInner(Fraction(howMuch), simulate)
	}

	fun receiveEnergyInner(howMuch: Long, simulate: Boolean): Fraction {
		return receiveEnergyInner(Fraction(howMuch), simulate)
	}

	fun receiveEnergyInner(howMuch: Int, simulate: Boolean): Fraction {
		return receiveEnergyInner(Fraction(howMuch), simulate)
	}

	val batteryLevel: Fraction
	val maxBatteryLevel: Fraction
	val missingPower: Fraction
		get() = (maxBatteryLevel - batteryLevel).moreThanZero()

	override fun receiveEnergy(maxReceive: Int, simulate: Boolean): Int {
		val received = receiveEnergyOuter(maxReceive, true).toInt()

		// Receiving only a fraction
		if (received == 0)
			return 0

		return receiveEnergyOuter(Fraction(received), simulate).toInt()
	}

	override fun extractEnergy(maxReceive: Int, simulate: Boolean): Int {
		val extracted = extractEnergyOuter(maxReceive, true).toInt()

		// Extracting only a fraction
		if (extracted == 0)
			return 0

		return extractEnergyOuter(Fraction(extracted), simulate).toInt()
	}

	override fun getEnergyStored(): Int {
		val level = batteryLevel

		if (level < MatteryCapability.INT_MAX_VALUE)
			return level.toInt()

		return Int.MAX_VALUE
	}

	override fun getMaxEnergyStored(): Int {
		val level = maxBatteryLevel

		if (level < MatteryCapability.INT_MAX_VALUE)
			return level.toInt()

		return Int.MAX_VALUE
	}

	override fun canExtract(): Boolean {
		return extractEnergyOuter(Fraction.ONE, true) > Fraction.ZERO
	}

	override fun canReceive(): Boolean {
		return receiveEnergyOuter(Fraction.ONE, true) > Fraction.ZERO
	}
}