package ru.dbotthepony.mc.otm.capability

import net.minecraft.nbt.CompoundTag
import net.minecraft.world.level.block.entity.BlockEntity
import net.minecraftforge.common.util.INBTSerializable
import ru.dbotthepony.mc.otm.core.Fraction
import ru.dbotthepony.mc.otm.core.Fraction.Companion.deserializeNBT
import ru.dbotthepony.mc.otm.ifHas
import ru.dbotthepony.mc.otm.set

open class MatteryMachineEnergyStorage @JvmOverloads constructor(
	protected val listener: () -> Unit,
	val type: MachineType,
	override var maxBatteryLevel: Fraction = DEFAULT_MAX_CAPACITY,
	protected var maxInput: Fraction = DEFAULT_MAX_RECEIVE,
	protected var maxOutput: Fraction = maxInput
) : IMatteryEnergyStorage, INBTSerializable<CompoundTag> {
	@JvmOverloads
	constructor(
		listener: BlockEntity,
		type: MachineType,
		maxBatteryLevel: Fraction = DEFAULT_MAX_CAPACITY,
		maxInput: Fraction = DEFAULT_MAX_RECEIVE,
		maxOutput: Fraction = maxInput) : this({listener.setChanged()}, type, maxBatteryLevel, maxInput, maxOutput)

	enum class MachineType {
		WORKER, GENERATOR, CAPACITOR
	}

	override var batteryLevel = Fraction.ZERO
		protected set

	override fun extractEnergyOuter(howMuch: Fraction, simulate: Boolean): Fraction {
		if (type == MachineType.WORKER)
			return Fraction.ZERO

		return extractEnergyInner(howMuch, simulate)
	}

	override fun extractEnergyInner(howMuch: Fraction, simulate: Boolean): Fraction {
		val new = batteryLevel.minus(howMuch.min(maxOutput)).moreThanZero()
		val diff = batteryLevel.minus(new)

		if (!simulate && batteryLevel != new) {
			batteryLevel = new
			listener()
		}

		return diff
	}

	override fun receiveEnergyOuter(howMuch: Fraction, simulate: Boolean): Fraction {
		if (type == MachineType.GENERATOR)
			return Fraction.ZERO

		return receiveEnergyInner(howMuch, simulate)
	}

	override fun receiveEnergyInner(howMuch: Fraction, simulate: Boolean): Fraction {
		val new = batteryLevel.plus(howMuch.min(maxInput)).min(maxBatteryLevel)
		val diff = new.minus(batteryLevel)

		if (!simulate && batteryLevel != new) {
			batteryLevel = new
			listener()
		}

		return diff
	}

	override fun canExtract(): Boolean {
		return type != MachineType.WORKER
	}

	override fun canReceive(): Boolean {
		return type != MachineType.GENERATOR
	}

	override fun serializeNBT(): CompoundTag {
		val tag = CompoundTag()
		// TODO: А это вообще надо?
		tag["energy_stored"] = batteryLevel.serializeNBT()

		// tag["energy_stored_max"] = maxBatteryLevel.serializeNBT()
		// tag["max_input"] = maxInput.serializeNBT()
		// tag["max_output"] = maxOutput.serializeNBT()

		return tag
	}

	override fun deserializeNBT(nbt: CompoundTag) {
		nbt.ifHas("energy_stored") { batteryLevel = deserializeNBT(it) }
		// nbt.ifHas("energy_stored_max") { maxBatteryLevel = deserializeNBT(it) }
		// nbt.ifHas("max_input") { maxInput = deserializeNBT(it) }
		// nbt.ifHas("max_output") { maxOutput = deserializeNBT(it) }
	}

	companion object {
		val DEFAULT_MAX_RECEIVE = Fraction(200)
		val DEFAULT_MAX_EXTRACT = Fraction(200)
		val DEFAULT_MAX_CAPACITY = Fraction(60000)
	}
}